/* global require, cbrequire, requirejs, define, console: false */
/**
 * @package     Rovexo.Plugin
 * @subpackage  System.jrequire
 *
 * @copyright   Copyright (C) 2018 Rovexo. All rights reserved.
 * @license		GNU General Public License version 2 or later OR MIT
 */
(function (window, document, require) {

	"use strict";

	// Fire the custom event for other JS to know we're ready
	var event = document.createEvent('Event');
	event.initEvent('requirejs_loaded', true, true);
	document.dispatchEvent(event);

	// Prepare the app config object (see info below)
	var appConfig = {};

	// On the requireJS script tag we made a data attribute with settings from the server
	// This serves as the 'gateway' for passing server data without using a separate xhr call
	if (document.getElementById('jrequire-require-tag')) {
		appConfig = JSON.parse(document.getElementById('jrequire-require-tag').dataset.appConfig);
	}
	// We don't load requireJS in case other software already loaded requirejs. In this case
	// we add the the main.js script tag with the same JSON data. This loads the appConfig from there.
	else {
		if(document.getElementById('jrequire-main-file-tag')) {
			appConfig = JSON.parse(document.getElementById('jrequire-main-file-tag').dataset.appConfig);
		}
	}

	var configuration = {
		baseUrl: appConfig.urlBase,
		paths: appConfig.customPaths,
		shim: appConfig.customShims
	};

	require.config(configuration);

	var moduleCall, moduleId, call;

	for (var i in appConfig.calls) {
		if (appConfig.calls.hasOwnProperty(i)) {

			moduleCall = appConfig.calls[i];

			// If we got a ::, then split between moduleId and call, require the module and call the method
			if (moduleCall.split('::', 2).length === 2) {

				moduleId = moduleCall.split('::', 2)[0];
				call = moduleCall.split('::', 2)[1];

				require([moduleId], function(module) {

					if (typeof(module[call]) !== 'function') {
						window.alert('Module call ' + moduleCall + ' seems faulty. Module does not have a function called "' + call + '"');
						return;
					}

					module[call].apply(module);

				});

			}
			// Otherwise just require the module
			else {
				require([moduleCall]);
			}

		}
	}

})(window, document, require);